library robomotion_app_lib;

import 'package:flutter/widgets.dart';
import 'package:dio/dio.dart';
import 'package:robomotion_app_lib/models/published_flow.dart';
import 'package:web_socket_channel/io.dart';

Dio _dio = Dio();

class Robomotion {
  final Config config;
  final Widget app;
  final Function(dynamic data) onMsgRecv;
  String _apiURL;
  String _appURL;
  IOWebSocketChannel _channel;

  Robomotion({@required this.config, @required this.app, @required this.onMsgRecv}) {
    _apiURL = _getAPIUrl();
    _appURL = _getAppUrl();
  }

  Future<void> run() async {
    await _connectAppServer();
    runApp(this.app);
  }

  Future<List<PublishedFlow>> getPublishedFlows() async {
    String url = "${this._apiURL}/v1/flows.published.list";
    Map resp = await _request(method: "GET", url: url, queryParams: {"page": 1, "size": 100, "search": ""});

    List<PublishedFlow> flows = [];
    for (dynamic f in resp["flows"]) {
      flows.add(PublishedFlow.fromJson(f));
    }
    return flows;
  }

  Future<Map> _request({@required String method, @required url, Map body, Map<String, dynamic> queryParams}) async {
    final resp = await _dio.request(url,
        queryParameters: queryParams,
        options: Options(method: method, headers: {
          'Authorization': 'Bearer ${this.config.token}',
          'Content-Type': 'application/json; charset=UTF-8',
        }),
        data: body);

    if (!resp.data.containsKey("ok") || !resp.data["ok"]) {
      throw new Exception("Request failed: ${resp.data.toString()}");
    }

    return resp.data;
  }

  Future<void> _connectAppServer() async {
    _channel = IOWebSocketChannel.connect("$_appURL?id=${config.id}&sid=${config.token}");
    _readChannel();
  }

  Future<void> _readChannel() async {
    try {
      await for (dynamic data in _channel.stream) {
        onMsgRecv(data);
      }
    } catch (ex) {
      _channel.sink.close();
    }
  }

  void sendAppMsg(dynamic data) {
    _channel.sink.add(data);
  }

  String _getAPIUrl() {
    String ws = this.config.workspace;
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0];
    return domain == "robomotion.io" ? "$protocol//api.$domain" : "$ws/api";
  }

  String _getAppUrl() {
    String ws = this.config.workspace;
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0] == "https" ? "wss" : "ws";
    return domain == "robomotion.io" ? "$protocol//apps.$domain" : "$protocol//$domain/app/app";
  }
}

class Config {
  final String id;
  final String namespace;
  final String token;
  final String repositoryID;
  final String workspace;

  Config(
      {@required this.id,
      @required this.namespace,
      @required this.token,
      @required this.repositoryID,
      @required this.workspace});
}
